@@ -24,7 +24,7 @@
 #include "html.h"
 #include "HTMLControl.h"
 #include "HTMLDriver.h"
-#include "HTMLParser.tab.hh"
+#include "HTMLParser.hh"
 #include "sgml.h"
 #include "cmp_nocase.h"
 #include "istr.h"
@@ -62,6 +62,109 @@ HTMLControl::htmlparser_yyerror(const char *p)
 		<< std::endl;
 }
 
+struct htmlparsertoken *HTMLControl::get_nth_token(int id)
+{
+	int i;
+	htmlparsertoken *tokw;
+	htmlparsertoken *tail = NULL;
+
+	for (i = 0, tokw = next_tokens; tokw != NULL; tokw = tokw->next, i++) {
+		if (i == id)
+			return tokw;
+		tail = tokw;
+	}
+
+	if (i == id) {
+		if (tail == NULL)
+			next_tokens = tokw = new htmlparsertoken;
+		else
+			tokw = tail->next = new htmlparsertoken;
+		tokw->next_token = yylex2(&tokw->next_token_value,
+								  &tokw->next_token_tag_type);
+		tokw->next = NULL;
+
+		/* do some space normalisation on PCDATA here to ease whitespace
+		 * logics lateron */
+		if (tokw->next_token == HTMLParser_token::PCDATA && !literal_mode) {
+			istr &s(*tokw->next_token_value.strinG);
+			/* empty string, just skip, get the next one */
+			if (s.empty()) {
+				del_nth_token(id);
+				return get_nth_token(id);
+			}
+
+			/* we got something, collate sequences of whitespace
+			 * within the string */
+			for (string::size_type x = 0; x < s.length(); x++) {
+				if (isspace(s[x])) {
+					string::size_type y;
+					for (y = x + 1; y < s.length() && isspace(s[y]); y++)
+						;
+					s.replace(x, y - x, " ");
+				}
+			}
+
+			/* if we have trailing or leading spaces, transform them
+			 * into their own token, so collapsing them is easier */
+			if (s.length() > 1) {
+				if (isspace(s[s.length() - 1])) {
+					s.erase(s.length() - 1, 1);  /* drop space */
+					/* append token */
+					htmlparsertoken *space = new htmlparsertoken;
+					space->next_token = HTMLParser_token::PCDATA;
+					space->next_token_tag_type = NOT_A_TAG;
+					space->next_token_value.strinG = new istr(" ");
+					space->next = NULL;
+					tokw->next = space;
+				}
+				if (isspace(s[0])) {
+					s.erase(0, 1);  /* drop space */
+					/* prepend token */
+					htmlparsertoken *space = new htmlparsertoken;
+					space->next = tokw;
+					if (next_tokens == tokw) {
+						next_tokens = space;
+					} else {
+						tail->next = space;
+					}
+					space->next_token = HTMLParser_token::PCDATA;
+					space->next_token_tag_type = NOT_A_TAG;
+					space->next_token_value.strinG = new istr(" ");
+					tokw = space;
+				}
+			}
+		}
+
+		return tokw;
+	}
+
+	return NULL;
+}
+
+void HTMLControl::del_nth_token(int id)
+{
+	int i;
+	htmlparsertoken *tokw;
+	htmlparsertoken *tail = NULL;
+
+	for (i = 0, tokw = next_tokens; tokw != NULL; tokw = tokw->next, i++) {
+		if (i == id)
+			break;
+		tail = tokw;
+	}
+
+	if (i == id) {
+		if (tail == NULL) {
+			next_tokens =  next_tokens->next;
+		} else {
+			tail->next = tokw->next;
+		}
+		if (tokw->next_token == HTMLParser_token::PCDATA)
+			delete tokw->next_token_value.strinG;
+		delete tokw;
+	}
+}
+
 /*
  * Effectively, this method simply invokes "yylex2()", but it does some
  * postprocessing on PCDATA tokens that would be difficult to do in "yylex2()".
@@ -70,23 +173,30 @@ int HTMLControl::htmlparser_yylex(
 		html2text::HTMLParser::semantic_type *value_return)
 {
 	for (;;) { // Notice the "return" at the end of the body!
-		int token, tag_type;
+		int token;
+		int tag_type;
+		int next_token;
+		int next_token_tag_type;
+		html2text::HTMLParser::semantic_type next_token_value;
+		struct htmlparsertoken *firsttok;
 
-		if (next_token == EOF) {
-			token = yylex2(value_return, &tag_type);
-		} else {
-			token = next_token;
-			*value_return = next_token_value;
-			tag_type = next_token_tag_type;
-			next_token = EOF;
-		}
+		firsttok      = get_nth_token(0);
+		token         = firsttok->next_token;
+		tag_type      = firsttok->next_token_tag_type;
+		*value_return = firsttok->next_token_value;
+
+		/* pop token */
+		next_tokens = next_tokens->next;
+		delete firsttok;
 
 		/* Switch on/off "literal mode" on "<PRE>" and "</PRE>" */
 		if (token == HTMLParser_token::PRE) {
 			literal_mode = true;
 
-			if (next_token == EOF)
-				next_token = yylex2(&next_token_value, &next_token_tag_type);
+			firsttok            = get_nth_token(0);
+			next_token          = firsttok->next_token;
+			next_token_value    = firsttok->next_token_value;
+
 			if (next_token == HTMLParser_token::PCDATA) {
 				/* Swallow '\n' immediately following "<PRE>" */
 				istr &s(*next_token_value.strinG);
@@ -98,79 +208,74 @@ int HTMLControl::htmlparser_yylex(
 		if (token == HTMLParser_token::END_PRE)
 			literal_mode = false;
 
+		if (tag_type == BLOCK_START_TAG ||
+			tag_type == BLOCK_END_TAG ||
+			token == HTMLParser_token::BR ||
+			token == HTMLParser_token::HR)
+			document_start = true;  /* swallow whitespace */
+
 		if (token == HTMLParser_token::PCDATA) {
-			/* In order to post-process the PCDATA token, we need to
-			 * look ahead one token...  */
-			if (next_token == EOF)
-				next_token = yylex2(&next_token_value, &next_token_tag_type);
+			if (literal_mode) {
+				firsttok            = get_nth_token(0);
+				next_token          = firsttok->next_token;
+				next_token_value    = firsttok->next_token_value;
 
-			/* Erase " '\n' { ' ' } " immediately before "</PRE>".  */
-			if (next_token == HTMLParser_token::END_PRE) {
-				istr &s(*value_return->strinG);
-				string::size_type x = s.length();
-				while (x > 0 && s[x - 1] == ' ')
-					--x;
-				if (x > 0 && s[x - 1] == '\n')
-					s.erase(x - 1, string::npos);
-			}
-			/* Erase whitespace before end tag or block start tag. */
-			else if (!literal_mode && (
-					next_token_tag_type == END_TAG ||
-					next_token_tag_type == BLOCK_END_TAG ||
-					next_token_tag_type == BLOCK_START_TAG
-					))
-			{
-				istr &s(*value_return->strinG);
-				string::size_type x = s.length();
-				while (x > 0 && isspace(s[x - 1]))
-					--x;
-				s.erase(x, string::npos);
-			}
+				/* Erase " '\n' { ' ' } " immediately before "</PRE>".  */
+				if (next_token == HTMLParser_token::END_PRE) {
+					istr &s(*value_return->strinG);
+					string::size_type x = s.length();
+					while (x > 0 && s[x - 1] == ' ')
+						--x;
+					if (x > 0 && s[x - 1] == '\n')
+						s.erase(x - 1, string::npos);
+				}
+			} else {
+				/* In order to post-process the PCDATA token, we need to
+			 	 * look ahead until we find another PCDATA token.  We're
+			 	 * trying to eliminate empty ones here, so as to avoid too
+			 	 * little and too much whitespace.  During reading,
+			 	 * strings are whitespace normalised, and trailing or
+			 	 * leading spaces are converted into separate elements
+			 	 * here. */
 
-			/* Collate sequences of whitespace, if not in "literal mode". */
-			if (!literal_mode) {
 				istr &s(*value_return->strinG);
-				for (string::size_type x = 0; x < s.length(); ++x) {
-					if (isspace(s[x])) {
-						string::size_type y;
-						for (y = x + 1; y < s.length() && isspace(s[y]); ++y)
-							;
-						s.replace(x, y - x, " ");
+				bool keepspace = document_start ? false : true;
+				if (isspace(s[0])) {  /* this must be a space on its own */
+					for (int i = 0; ;) {
+						firsttok            = get_nth_token(i);
+						next_token          = firsttok->next_token;
+						next_token_tag_type = firsttok->next_token_tag_type;
+						next_token_value    = firsttok->next_token_value;
+
+						if (next_token == EOF ||
+							next_token == HTMLParser_token::PRE ||
+							next_token_tag_type == BLOCK_START_TAG ||
+							next_token_tag_type == BLOCK_END_TAG)
+						{
+							keepspace = false;
+							break;
+						}
+
+						if (next_token == HTMLParser_token::PCDATA) {
+							istr &ns(*next_token_value.strinG);
+							if (isspace(ns[0])) {
+								/* duplicate, drop */
+								del_nth_token(i);
+								continue;
+							} else {
+								break;  /* we're done here */
+							}
+						}
+						i++;
 					}
-				}
-				if (s.empty()) {
-					delete value_return->strinG;
-					continue;
-				}
-			}
-		}
 
-		/* Erase whitespace after start tag or block end tag, if not in
-		 * "literal mode".  */
-		if (!literal_mode &&
-				((
-				  tag_type == START_TAG ||
-				  tag_type == BLOCK_START_TAG ||
-				  tag_type == BLOCK_END_TAG ||
-				  token == HTMLParser_token::BR ||
-				  token == HTMLParser_token::HR
-				 ) &&
-				 token != HTMLParser_token::SCRIPT &&
-				 token != HTMLParser_token::STYLE
-				))
-		{
-			if (next_token == EOF)
-				next_token = yylex2(&next_token_value, &next_token_tag_type);
-			if (next_token == HTMLParser_token::PCDATA) {
-				istr &s(*next_token_value.strinG);
-				string::size_type x;
-				for (x = 0; x < s.length() && isspace(s[x]); ++x)
-					;
-				if (x > 0)
-					s.erase(0, x);
-				if (s.empty()) {
-					delete next_token_value.strinG;
-					next_token = EOF;
+					if (!keepspace) {
+						/* drop, we're excessive */
+						delete value_return->strinG;
+						continue;
+					}
+				} else {
+					document_start = false;
 				}
 			}
 		}
@@ -362,7 +467,7 @@ HTMLControl::yylex2(html2text::HTMLParser::semantic_type *value_return,
 				return HTMLParser_token::DOCTYPE;
 			}
 
-			if (c == '/' || isalpha(c) || c == '_') {
+			if (c == '/' || isalpha(c) || c == '_' || c == '?') {
 				string tag_name;
 				bool is_end_tag = false;
 
@@ -370,6 +475,9 @@ HTMLControl::yylex2(html2text::HTMLParser::semantic_type *value_return,
 					is_end_tag = true;
 					c = get_char();
 				}
+				if (c == '?') {
+					c = get_char();
+				}
 				if (!isalpha(c) && c != '_')
 					return HTMLParser_token::SCAN_ERROR;
 				tag_name += c;
@@ -558,7 +666,7 @@ HTMLControl::yylex2(html2text::HTMLParser::semantic_type *value_return,
 				if (c == '<') {
 					int c2;
 					unget_char(c2 = get_char());
-					if (c2 == '!' || c2 == '/' || isalpha(c2)) {
+					if (c2 == '!' || c2 == '/' || c2 == '?' || isalpha(c2)) {
 						unget_char(c);
 						break;
 					}
