@@ -5,6 +5,8 @@
  *
  * Author: Arno Unkrig <arno@unkrig.de>
  *
+ * Copyright 2020-2023 Fabian Groffen <grobian@gentoo.org>
+ *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
  * the Free Software Foundation; either version 2 of the License, or
@@ -52,6 +54,8 @@ text.\n\
   -to_encoding   Output using given encoding\n\
   -ascii         Use plain ASCII for output instead of UTF-8\n\
                  alias for: -to_encoding ASCII//TRANSLIT \n\
+  -utf8          Assume both terminal and input stream are in UTF-8 mode\n\
+                 alias for: -from_encoding UTF-8 -to_encoding UTF-8 \n\
   -o <file>      Redirect output into <file>\n\
 ";
 
@@ -123,6 +127,9 @@ main(int argc, char **argv)
 			extarg = &to_encoding;
 		} else if (!strcmp(arg, "-ascii")) {
 			to_encoding = "ASCII//TRANSLIT";  /* create things like (c) */
+		} else if (!strcmp(arg, "-utf8")) {
+			from_encoding = "UTF-8";
+			to_encoding = "UTF-8";
 		} else {
 			std::cerr
 				<< "Unrecognized command line option \""
@@ -189,9 +196,12 @@ main(int argc, char **argv)
 
 	{
 		std::ifstream ifs;
+		std::string homerc;
 
-		if (rcfile == NULL && home != NULL)
-			rcfile = (string(home) + "/.html2textrc").c_str();
+		if (rcfile == NULL && home != NULL) {
+			homerc = string(home) + "/.html2textrc";
+			rcfile = homerc.c_str();
+		}
 		if (rcfile != NULL)
 			ifs.open(rcfile);
 		if (rcfile == NULL || !ifs.rdbuf()->is_open())
@@ -241,9 +251,9 @@ main(int argc, char **argv)
 
 		if (driver.parse() != 0)
 			exit(1);
-	}
 
-	is.close();
+		is.close();
+	}
 
 	return 0;
 }
