@@ -1,5 +1,5 @@
 /*
- * Copyright 2020-2022 Fabian Groffen <grobian@gentoo.org>
+ * Copyright 2020-2023 Fabian Groffen <grobian@gentoo.org>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -33,6 +33,11 @@ class iconvstream {
 			fd_os(-1)
 		{
 		}
+		~iconvstream()
+		{
+			if (find_tokens_c_str_buf != nullptr)
+				delete[] find_tokens_c_str_buf;
+		}
 
 		void open_is(const char *url, const char *encoding);
 		void open_is(const string &url, const char *encoding);
@@ -61,27 +66,30 @@ class iconvstream {
 		iconvstream &operator<<(char inp);
 
 	private:
+		const char* find_tokens(char*, size_t, const char**);
+		char *find_tokens_c_str_buf = nullptr;
+
 		const char *open_err;
 		const char *encoding;
 
 		int fd_is;
-		iconv_t iconv_handle_is;
-		unsigned char *readbuf;
+		iconv_t iconv_handle_is = nullptr;
+		unsigned char *readbuf = nullptr;
 		size_t readbufsze;
 		size_t readbuflen;
 		size_t readbufpos;
-		unsigned char *rutf8buf;
+		unsigned char *rutf8buf = nullptr;
 		size_t rutf8bufsze;
 		size_t rutf8buflen;
 		size_t rutf8bufpos;
 
 		int fd_os;
-		iconv_t iconv_handle_os;
-		unsigned char *writebuf;
+		iconv_t iconv_handle_os = nullptr;
+		unsigned char *writebuf = nullptr;
 		size_t writebufsze;
 		size_t writebuflen;
 		size_t writebufpos;
-		unsigned char *wutf8buf;
+		unsigned char *wutf8buf = nullptr;
 		size_t wutf8bufsze;
 		size_t wutf8buflen;
 		size_t wutf8bufpos;
